#!/usr/bin/newlisp

(set 'Version "0.2.1")

;; Copyright (c) 2009, Greg Slepak
;; All rights reserved.
;; 
;; Redistribution and use in source and binary forms, with or without modification,
;; are permitted provided that the following conditions are met:
;; 
;;     * Redistributions of source code must retain the above copyright notice
;;       this list of conditions and the following disclaimer.
;;     * Redistributions in binary form must reproduce the above copyright notice,
;;       this list of conditions and the following disclaimer in the documentation
;;       and/or other materials provided with the distribution.
;;     * Neither the name of Tao Effect nor the names of its contributors
;;       may be used to endorse or promote products derived from this software
;;       without specific prior written permission.
;; 
;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;; DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
;; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
;; CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
;; OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
;; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

; list containing undo actions to perform on error
(set 'undo '())

; mark Functions

(define (error-out)
	(apply println (cons "ERROR: " (args)))
	
	; if necessary, undo the actions in the opposite order (most recent first)
	(while (set 'undo-item (pop undo))
		(println "Reverting with action: " undo-item)
		
		; evaulate the undo action and capture its return value
		(set 'rval (eval undo-item))
		
		; check for error from built-in function or from executed command
		(when (or (nil? rval) (and (number? rval) (!= 0 rval)))
			(println "FAILED TO UNDO: " undo-item " => " rval)
			(if undo (println "Undo actions not finished:\n" (join (map string undo) "\n")))
			(exit 1)
		)
	)
	(exit 1)
)

(define (warn)
	(apply println (cons "WARNING: " (args)))
)

(define (usage)
	(println "ebswitch version " Version)
	(println "Usage: " (main-args 1) " twitter_profile_name")
	(exit 1)
)

(define (undoable-rename from to)
	(if-not (rename-file from to)
		(error-out "Couldn't rename: '" from "' -> '" to "'"))
	(push (list 'rename-file to from) undo)
)

(define (undoable-copy from to)
	(if-not (zero? (! (append {cp -R "} from {" "} to {"})))
		(error-out "Couldn't copy: '" from "' -> '" to "'"))
	
	; to undo a copy we delete what was copied
	(push (list 'remove-forcefully to) undo)
)

(define (remove-forcefully file)
	 (! (append {rm -rf "} file {"}))
)

(define (osa str) 
	(exec (format {osascript -e '%s' 2>&1} str))
)

; mark Sanity Checks

(if-not (env "HOME")
	(error-out "HOME environment var not defined!"))

(if-not (set 'switch-to-username (main-args 2))
	(usage))
	
(if (= "-fresh" switch-to-username)
	(error-out "Username cannot be " switch-to-username))

(when (exec "ps x | grep EventBox | grep -v grep")
	(print "Quit EventBox? [y|n]: ")
	(if-not (= "y" (read-line))
		(error-out "Please close EventBox first"))
	(set 'rval (osa {tell application "EventBox" to quit}))
	(if-not (null? rval)
		(error-out "Couldn't quit EventBox: " (join rval "\n")))
	; sleep two seconds after it quits, otherwise EB won't relaunch at the end of the script
	; and there can be problems writing the defaults
	(sleep 2000)
)

(set 'EB_SUPPORT_DIR (append (env "HOME") "/Library/Application Support/EventBox"))
(set 'EB_PREFS_FILE  (append (env "HOME") "/Library/Preferences/com.thecosmicmachine.EventBox.plist"))

(if-not (file? EB_SUPPORT_DIR)
	(error-out "EB support dir doesn't exist"))
(if-not (file? EB_PREFS_FILE)
	(error-out "EB preferences don't exist"))

; mark Get current twitter profile name

(regex {com.thecosmicmachine.twitterplugin.*Username = (.*?);} (join (exec "defaults read com.thecosmicmachine.EventBox pluginAccounts")))

; check for nil or otherwise bad input
(if (string? $1)
	(set 'switch-from-username (trim $1 {"})) ; sometimes for some reason the username is surrounded by quotes
	(set 'switch-from-username "") ; will be caught by the null? check below
)

(if (= switch-from-username switch-to-username)
	(error-out "Can't switch, profiles both use username: " switch-to-username))

(println switch-from-username " => " switch-to-username)

(set 'prev-sup-folder (append EB_SUPPORT_DIR "_" switch-from-username))
(set 'prev-pref-file (append EB_PREFS_FILE "_" switch-from-username))
(set 'new-sup-folder (append EB_SUPPORT_DIR "_" switch-to-username))
(set 'new-pref-file (append EB_PREFS_FILE "_" switch-to-username))

(if (null? switch-from-username)
	(if (and (file? new-sup-folder) (file? new-pref-file))
		(begin
			(print "No twitter profile currently set.\nDelete current settings? [y|n]: ")
			(if-not (= "y" (read-line))
				(error-out "Please set account settings for current profile"))
			(remove-forcefully EB_PREFS_FILE)
			(remove-forcefully EB_SUPPORT_DIR)
		)
		(error-out "No twitter profile currently set.\nPlease set account settings for it.")
	)
)

; whether we should delete the previously known tweets from the old account
(set 'fresh-account (not (file? new-sup-folder)))

; mark Go!

; check again in case it was removed because of the code above
(if (file? EB_SUPPORT_DIR) (undoable-rename EB_SUPPORT_DIR prev-sup-folder))
(if (file? EB_PREFS_FILE) (undoable-rename EB_PREFS_FILE prev-pref-file))

; if the support directory that we're switching to exists, simply rename it
; otherwise we'll copy the contents of the profile that we are switching from
; so that the user can then setup the new account
(if (file? new-sup-folder)
	(undoable-rename new-sup-folder EB_SUPPORT_DIR)
	(undoable-copy prev-sup-folder EB_SUPPORT_DIR)
)

(if (file? new-pref-file)
	(undoable-rename new-pref-file EB_PREFS_FILE)
	(undoable-copy prev-pref-file EB_PREFS_FILE)
)

; no more error-out's past this point!

(when fresh-account
	(println "Creating fresh account for: " switch-to-username)
	(if-not (zero? (remove-forcefully (append EB_SUPPORT_DIR "/com.thecosmicmachine.twitterplugin")))
		(warn "failed to delete twitter history")
	)
	(if-not (zero? (! "defaults delete com.thecosmicmachine.EventBox pluginAccounts"))
		(warn "failed to remove account login info")
	)
	(if-not (zero? (! "defaults delete com.thecosmicmachine.EventBox com.thecosmicmachine.twitterplugin"))
		(warn "failed to remove account settings")
	)
)

(println "Successfully switched to profile: " switch-to-username)
(! "open -b com.thecosmicmachine.EventBox")
(exit)
